import logging; log = logging.getLogger('soappublisher.publication')
import sys
from zope import interface, proxy
from zope.app.publication.interfaces import ISOAPRequestFactory
from zope.publisher.http import HTTPRequest, HTTPResponse, DirectResult
from zope.security.proxy import isinstance
from ZSI import _get_element_nsuri_name
from ZSI import FaultFromException ,Fault
from ZSI.parse import ParsedSoap
from ZSI.writer import SoapWriter

from interfaces import ISOAPRequest

XMLPROLOG = '<?xml version="1.0" encoding="utf-8"?>'.encode('UTF-8')
class SOAPRequest(HTTPRequest):
    interface.implements(ISOAPRequest)
    
    parsedSoap = None
    _args = ()
    
    def _createResponse(self):
        """ Create a specific SOAP response object """
        return SOAPResponse()
    
    def processInputs(self):
        # NOTE - we don't deal with multipart SOAP requests here.

        # XXX using readlines() instead of lines()
        # as twisted's BufferedStream sends back
        # an empty stream here for read() (bug)
        xml = ''.join(self._body_instream.readlines())
        self.parsedSoap = ps = ParsedSoap(xml)
        
        # Find the method/action.
        namespace, action = _get_element_nsuri_name(ps.body_root)

        # Translate '.' to '/' in function to represent object traversal.
        action = action.split('.')

        if action:
            self.setPathSuffix(action)

@interface.implementer(ISOAPRequestFactory)
def SOAPRequestFactory(input_stream, env):
    return SOAPRequest(input_stream, env)

class SOAPResponse(HTTPResponse):
    def setResult(self, result):
        """ Result should be a SOAP Response type-coded object """
        result = proxy.removeAllProxies(result)
        if isinstance(result, Fault):
            # Convert fault object to SOAP response
            body = result.AsSOAP()
            self.setStatus(500)
        else:
            sw = SoapWriter()
            try:
                if isinstance(result, basestring):
                    log.info('SETRESULT: %s', result)
                body = str(sw.serialize(result))
            except:
                log.exception('Exception while serializing in setResult')
                self.handleException(sys.exc_info())
                return
        
        if not body.startswith('<?xml'):
            body = XMLPROLOG+body
        super(SOAPResponse, self).setResult(DirectResult(
            (body,),
            [('content-type', 'text/xml; charset=utf-8'),
             ('content-length', str(len(body))),
             ('connection', 'close')]
        ))

    def handleException(self, exc_info):
        etype, evalue, etb = exc_info[:]
        if isinstance(evalue, Fault):
            fault = evalue
        else:
            fault = FaultFromException(evalue, False, etb)
        self.setResult(fault)
