""" SOAP Publisher / 'Views' """
from zope import component, interface, location

from interfaces import (ISOAPRequest, ISOAPPublisher, ISOAPView, 
    ISOAPMethodPublisher)

class SOAPView(object):
    """ 
    A base SOAP view class that can be used as mixin or base for SOAP views.
    """
    interface.implements(ISOAPView)

    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.ps = getattr(request, 'parsedSoap', None)

class SOAPMethodPublisher(SOAPView, location.Location):
    """ 
    Base class for very simple SOAP views that publish methods. Allows parent
    rebinding, falling back on the view's context as parent.
    """
    interface.implements(ISOAPMethodPublisher)

    def __getParent(self):
        return hasattr(self, '_parent') and self._parent or self.context

    def __setParent(self, parent):
        self._parent = parent

    __parent__ = property(__getParent, __setParent)


class SOAPMethodTraverser(object):
    interface.implements(ISOAPPublisher)
    component.adapts(ISOAPMethodPublisher, ISOAPRequest)

    def __init__(self, context, request):
        self.context = context
        
    def publishTraverse(self, request, name):
        """ Traverse directly to the named method, which may be on self """
        return getattr(self.context, name)
